(ns sandbar.examples.fancy-form-before
  "A form which will be made fancy by the addition of jQuery."
  (:use [ring.adapter.jetty :only [run-jetty]]
        [ring.middleware.file :only [wrap-file]]
        [hiccup.core :only [html]]
        [hiccup.page-helpers :only [doctype link-to]]
        [compojure.core :only [defroutes GET]]
        [sandbar.core :only [icon stylesheet javascript]]
        [sandbar.stateful-session :only [wrap-stateful-session
                                         flash-put!
                                         flash-get]]
        [sandbar.validation :only [add-validation-error
                                   build-validator
                                   non-empty-string]])
  (:require [compojure.route :as route]
            [sandbar.forms :as forms]
            [sandbar.examples.database :as db]))

(defn layout
  ([content] (layout content []))
  ([content javascripts]
     (html
      (doctype :html5)
      [:html
       [:head
        (stylesheet "sandbar.css")
        (stylesheet "sandbar-forms.css")
        (icon "icon.png")]
       [:body
        (if-let [m (flash-get :user-message)] [:div {:class "message"} m])
        [:h2 "Sandbar Form Example"]
        content]])))

(defn home []
  (layout
   [:div
    (link-to "/developer/edit" "Add Developer")
    [:table
     [:tr
      [:th "Name"] [:th "Hire Date"] [:th "Favorite Language"] [:th ""]]
     (map #(let [{:keys [id name hire-date language]} %]
             [:tr
              [:td name] [:td hire-date] [:td language]
              [:td (link-to (str "/developer/edit/" id) "Edit")]])
          (db/all-users))]]))

(def properties {:name "Name"
                 :hire-date "Hire Date"
                 :language "Language"})

(forms/defform fancy-form "/developer/edit"
  :fields [(forms/hidden :id)
           (forms/textfield :name)
           (forms/textfield :hire-date {:size 10})
           (forms/textfield :language)]
  :load #(db/find-user %)
  :on-cancel "/"
  :on-success #(do
                 (db/store-user %)
                 (flash-put! :user-message "Developer has been saved.")
                 "/")
  :validator #(non-empty-string % :name :hire-date :language properties)
  :properties properties)

(defroutes routes
  (fancy-form (fn [request form]
                (layout form)))
  (GET "/" [] (home))
  (route/not-found "<h1>Not Found</h1>"))

(def application-routes (-> routes
                            wrap-stateful-session
                            (wrap-file "public")))

(defn run []
  (run-jetty (var application-routes) {:join? false :port 8080}))
