(ns sandbar.examples.part-one.start
  (:use (ring.adapter jetty)
        (compojure core)
        (hiccup core page-helpers)))

(defn query [type]
  (str (name type) " data"))

(defn layout [content]
  (html
   [:html
    [:body
     [:h2 "Sandbar Authorization Example"]
     content]]))

(defn data-view [title data & links]
  [:div
   [:h3 title]
   [:p data]
   (if (seq links) links [:div (link-to "home" "Home")])])

(defn home-view []
  (data-view "Home"
             (query :public)
             [:div (link-to "member" "Member Data")]
             [:div (link-to "admin" "Admin Data")]))

(defn member-view []
  (data-view "Member Page"
             (query :members-only)))

(defn admin-view []
  (data-view "Admin Page"
             (query :top-secret)))

(defroutes my-routes
  (GET "/home*" [] (layout (home-view)))
  (GET "/member*" [] (layout (member-view)))
  (GET "/admin*" [] (layout (admin-view)))
  (ANY "*" [] (layout (home-view))))

(def app my-routes)

(defn run []
  (run-jetty (var app) {:join? false :port 8080}))
